<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Aug 28, 2013 3:05:22 PM
 * @version $Rev$
 */

/**
 * Job class
 * Used to ....
 */
class Entity_Source_Job extends Entity_Source {
	
	const FEED_URL_TEMPLATE = 'https://[appname].carerix.com/cxtools/wp_feed.php';
	protected $type = WP_Constants::ENTITY_TYPE_JOB;

	const EXCERPT_TEMPLATE = '[cx_intro_information heading="h4"]';
	const BODYTEXT_TEMPLATE = <<<EOT
[cx_intro_information heading="h4"]
<!--more-->
[cx_company_information heading="h4"]
[cx_vacancy_information heading="h4"]
[cx_requirements_information heading="h4"]
[cx_offer_information heading="h4"]
[cx_function_contact_information heading="h4"]
[cx_application_contact_information heading="h4"]
EOT;
	const FOOTERTEXT_TEMPLATE = "[cx_apply_button]";

	static function addDefaultJobSource()
	{
		$job_default_source_not_found = true;
		
		$sourcesTable = new WP_Table(Environment_Installation::TABLE_SOURCES);
		$jobSources = $sourcesTable->getByType(WP_Constants::ENTITY_TYPE_JOB);
		
		foreach($jobSources as $jobSource)
		{
			$settings = unserialize($jobSource['settings']);
			if(	isset($settings['is_default']) )
			{
				$job_default_source_not_found = false;
				break;
			}
		}
		
		// no default source for jobs was found nor any other job sources
		if($job_default_source_not_found && count($jobSources) == 0)
		{
			$data = self::getDefaultSettings();
			$jobSource = new Entity_Source_Job();
			$jobSource->save($data);
		}
		
		return $job_default_source_not_found;
	}
	
	/**
	 * 
	 * @return array
	 */
	static private function getDefaultSettings()
	{
		// find default form
		$data = array();
		$data['name'] = 'Default jobs source'; // Jobs Default Page
		
		if(false !== $form = self::getDefaultForm())
		{
			$data['apply_form'] = (int)$form['id'];
		}

		$data["excerpt"] = self::EXCERPT_TEMPLATE;
		$data["bodytext"] = self::BODYTEXT_TEMPLATE;
		$data["footer"] = self::FOOTERTEXT_TEMPLATE;
		$data['is_default'] = 'on';

		/**
		 * @deprecated
		 */
		$data['rss_link'] = 'on';
		$data['tell_a_friend_link'] = 'on';
		$data['print_link'] = 'on';
		$data['brief_section'] = 'on';
		$data['social_icons'] = 'on';
		$data['job_alert_subscription'] = 'on';

		return $data;
	}
	
	static private function getDefaultForm()
	{
		$formsTable = new WP_Table(Environment_Installation::TABLE_FORMS);
		$all_forms = $formsTable->getAll();
		
		foreach($all_forms as $form)
		{
			$form_settings = unserialize($form['settings']);
			
			if(isset($form_settings['is_default']))
			{
				return $form;
			}
		}
		return false;
	}

	/**
	 * @deprecated Unused?
	 * @return false|mixed
	 */
	static private function getAppName()
	{
		$settingsTable = new WP_Table(Environment_Installation::TABLE_SETTINGS);
		$app_name_rows = $settingsTable->getByKey('app_name');
		
		if(count($app_name_rows) >= 1)
		{
			return $app_name_rows[0]['value'];
		}
		return false;
	}

    public static function getAll() {
        $sourcesTable = new WP_Table(Environment_Installation::TABLE_SOURCES);
        $jobSources = $sourcesTable->getByType(WP_Constants::ENTITY_TYPE_JOB);
        foreach($jobSources as &$jobSource)
        {
            $jobSource['settings'] = unserialize($jobSource['settings']);
        }
        return $jobSources;
    }


	/**
	 * Migrate the fixed vacancy body text to a "shortcode" version. Admins can create their own layout in
	 * CXWP -> Sources -> panel "Publication Details Body"
	 *
	 * @since v2.23.0
	 */
	public static function upgradeToBodyTexts()
	{
		$jobSources = self::getAll();
		foreach($jobSources as $jobSource)
		{
			$migrate = strlen($jobSource["settings"]["bodytext"] ?? "") == 0;
			if ($migrate) {
				$id = $jobSource["id"];
				$jobSource["settings"]["name"] = $jobSource["name"];
				$jobSource["settings"]["bodytext"] = self::BODYTEXT_TEMPLATE;
				(new Entity_Source_Job($id))->save($jobSource["settings"]);
			}
		}
	}
}