<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Aug 29, 2013 1:48:30 PM
 * @version $Rev$
 */


/**
 * CronRunner class
 * Used to ....
 */
class Environment_CronRunner {
	
	protected $interface = null;
    protected $interfacer = null;
	protected $sourcesTable = null;
    private $post_id = 0;
    private $manual_called = false;
	
	public function __construct() {
		$this->interface = WP_Interfacer::getInstance();
        $this->interfacer = WP_Interfacer::getInstance();
		$this->sourcesTable = Table_Source::get(); // Environment_Installation::TABLE_SOURCES
        Environment_Installation::clear_categories();
        //$this->interfacer->taxonomy_clear_cache('category');
	}
	
	public function run( $manual_called = false )
	{
		$this->manual_called = $manual_called;
		/*
		 *  the cron objects each sets up a new category hierarchy, 
		 * so is best to clear wordpress taxonomy cache for category
		 */
		try {
            $this->interface->taxonomy_clear_cache('category');

            $this->runForNewsletters();

            $this->runForCompanies();

            $pageCandidates = 0;
            $hasMoreCandidates = true;
            while($hasMoreCandidates) {
                $hasMoreCandidates = $this->runForCandidates($pageCandidates);
                $pageCandidates += 1;
            }
            
            $this->runForPeople();

            $pageJobs = 0;
            $hasMoreJobs = true;
            while($hasMoreJobs){
                $hasMoreJobs = $this->runForJobs($pageJobs);
                $pageJobs += 1;
            }

            $this->runForPromotional();
            
            // delete lock file here
            return true;
        } 
        catch (Exception $ex)
        {
            return false;
        }
		return true;
	}

	/**
	 * starts the crons for each job source
	 *
	 * @param int $page
	 *
	 * @return bool
	 */
	public function runForJobs($page = 0)
	{
		$jobSources = $this->sourcesTable->getByType(WP_Constants::ENTITY_TYPE_JOB);
		// Utils_Log::info("\nEnvironment/CronRunner.php -> runForJobs()\n" . print_r($jobSources, true) . "\n\n");

		$hasMore = false;
		foreach($jobSources as $source)
		{
            $cronCleaner = new Utils_CronCleaner($source['id'], $this->manual_called);
            if(!$page) {
                $cronCleaner->clean();
            }

			$source['settings'] = unserialize($source['settings']);
			$_hasMore = $this->runJobCronForSource($source, $page);

            if($_hasMore === false) { // all the batches of items are processed
                $cronCleaner->run('job'); // Now old posts (=vacancies) are deleted that aren't in the feed-list anymore
	            $cronCleaner->setCxIsSyncedToThisDateTime( $source['id'] ); // set current datetime in "sync monitor file" (used for the next sync offset)
            }
            $hasMore = $hasMore || $_hasMore;
			$cronCleaner = null;
			unset($cronCleaner);
//			gc_collect_cycles();
		}
        // $cronCleaner = null;
        return $hasMore;
	}
	
	public function runJobCronForSource($source, $page = 0)
	{
        $hasMore = false;
        $jobCron = false;
		if($this->interface->wpml_is_active())
		{ // multi language
			$languages = $this->interface->get_wpml_active_languages();

			foreach($languages as $language => $__temp)
			{
				$jobCron = new Entity_Cron_Job($source, $this->manual_called);
                $jobCron->setPage($page);
				$jobCron->setRegisterLanguage($language);
				$moreNumber = $jobCron->run();  // retrieve jobs from feed and process them to WP
                if($moreNumber > 0)  $hasMore = true;
			}
            $__temp = null;
            $languages = null;
		}
		else // single language!
		{
			$jobCron = new Entity_Cron_Job($source, $this->manual_called);
            $jobCron->setPage($page);
			$moreNumber = $jobCron->run(); // retrieve jobs from feed and process them to WP
			if($moreNumber > 0) $hasMore = true;
		}
        
        if(!$hasMore) {
	        if($jobCron !== null) {
                $this->setPostID($jobCron->getPostID());
            }
            
            if($this->interface->wpml_is_active()) {
                $this->fixWPMLTrids();
            }
        }
		$jobCron = null;
		unset($jobCron);

        return $hasMore;
	}


    public function fixWPMLTrids() 
    {
        $languages = $this->interfacer->get_wpml_active_languages();
        foreach($languages as $language => $__temp)
        {
            $like = '_carerix_job_'.$language;
            $tableName = $this->interfacer->db_get_postmeta_name();
	        /** @noinspection SqlResolve */
	        $results = $this->interfacer->db_get_custom("select * from $tableName where `meta_key` = '$like'");
            foreach($results as $result) {

                if(!$this->interfacer->managed_by_carerix($result['post_id'])) {
                    continue;
                }
                
                $properties = $this->interfacer->entity_get_properties($result['post_id']);

                $trid = (string)$properties['vacancyID'][0];
                $trid = (int)$trid * 10000;
                $wpmlTable = $this->interfacer->wpml_table_name();
                $element_id = $result['post_id'];
	            /** @noinspection SqlResolve */
	            $selectSql = "SELECT * FROM $wpmlTable WHERE `element_id` = $element_id";

                $translations = $this->interfacer->db_get_custom($selectSql);
                if(isset($translations[0])) {
                    $translation = $translations[0];
                    $translation_id = $translation['translation_id'];

                    $this->interfacer->remove_ghost_translations($language, $trid, $translation_id);

	                /** @noinspection SqlResolve */
	                $updateSql = $this->interfacer->db()->prepare(
                        "UPDATE $wpmlTable
                            SET `language_code` = %s,
                                `trid` = %d
                            WHERE
                                `translation_id` = %s
                        ",
                        array(
                            $language,
                            $trid,
                            $translation_id
					));
                    
                    $this->interfacer->db()->query($updateSql);
                    // $this->interfacer->db()->print_error();
                } 
                
            }
        }
    }
    
    public function setPostID($post_id) {
        $this->post_id = $post_id;
    }
    
    public function getPostID() {
        return $this->post_id;
    }


    /**
     * @deprecated since v2.21.0
     *
	 * starts the crons for each promotional source
	 */
	public function runForPromotional()
	{
//		return false;

//		$promotionalSources = $this->sourcesTable->getByType(WP_Constants::ENTITY_TYPE_PROMOTIONAL);
//		foreach($promotionalSources as $source)
//		{
//			$source['settings'] = unserialize($source['settings']);
//			$promotionalCron = new Entity_Cron_Promotional($source);
//			$promotionalCron->run();
//
//            $cronCleaner = new Utils_CronCleaner($source['id']);
//            $cronCleaner->run('promotional');
//		}
	}
	
	/**
	 * 	@deprecated since v2.20.0
	 *
	 * starts the crons for each candidate source
	 */
	public function runForCandidates($page = 0)
	{
		return false;

//		$candidateSources = $this->sourcesTable->getByType(WP_Constants::ENTITY_TYPE_CANDIDATE);
//        $hasMore = false;
//		foreach($candidateSources as $source)
//		{
//            $cronCleaner = new Utils_CronCleaner($source['id']);
//            if(!$page) {
//                $cronCleaner->clean();
//            }
//			$source['settings'] = unserialize($source['settings']);
//			$_hasMore = $this->runCandidateCronForSource($source, $page);
//            $hasMore = $_hasMore || $hasMore;
//
//            if(!$hasMore) {
//                $cronCleaner = new Utils_CronCleaner($source['id']);
//                $cronCleaner->run('candidate');
//            }
//		}
//
//        return $hasMore;
	}

	/**
	 * @deprecated since v2.20.0
	 */
	public function runCandidateCronForSource($source, $page = 0)
	{
		return false;

//        $candidateCron = new Entity_Cron_Candidate($source);
//        $candidateCron->setPage($page);
//		$moreNumber = $candidateCron->run();
//        return $moreNumber > 0;
	}
	
	/**
	 * @deprecated since v2.20.0
	 *
	 * starts the crons for each people source
	 */
	public function runForPeople()
	{
		return;

//		$peopleSources = $this->sourcesTable->getByType(WP_Constants::ENTITY_TYPE_PEOPLE);
//		foreach($peopleSources as $source)
//		{
//			$source['settings'] = unserialize($source['settings']);
//			$this->runPeopleCronForSource($source);
//
//            $cronCleaner = new Utils_CronCleaner($source['id']);
//            $cronCleaner->run('people');
//		}

	}

	/**
	 * @deprecated since v2.20.0
	 */
	public function runPeopleCronForSource($source)
	{
		return;
//		$sourceCron = new Entity_Cron_People($source);
//		$sourceCron->run();
	}
	
	/**
	 * @deprecated since v2.20.0
	 *
	 * starts the crons for each company source
	 */
	public function runForCompanies()
	{
		return;
//		$companiesSources = $this->sourcesTable->getByType(WP_Constants::ENTITY_TYPE_COMPANY);
//		foreach ($companiesSources as $source)
//		{
//			$source['settings'] = unserialize($source['settings']);
//			$this->runCompanyCronForSource($source);
//		}
	}

	/**
	 * @deprecated since v2.20.0
	 *
	 */
	public function runCompanyCronForSource($source)
	{
		return;
//		$sourceCron = new Entity_Cron_Company($source);
//		$sourceCron->run();
	}
	
	/**
	 * starts the crons for each newsletter source
	 *
	 * @deprecated since v2.20.0
	 */
	public function runForNewsletters()
	{
		return;
//		$newsletterSources = $this->sourcesTable->getByType(WP_Constants::ENTITY_TYPE_NEWSLETTER);
//
//		foreach($newsletterSources as $source)
//		{
//			$source['settings'] = unserialize($source['settings']);
//			$this->runNewsletterCronForSource($source);
//		}
	}

	/**
	 * @param $source
	 *
	 * @deprecated since v2.20.0
	 */
	public function runNewsletterCronForSource($source)
	{
		return;
//		$sourceCron = new Entity_Cron_Newsletter($source);
//		$sourceCron->run();
	}
}