<?php

use Carerix\Api\Rest\Entity\CRPublication;

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Sep 3, 2013 4:27:07 PM
 * @version $Rev$
 */

class Frontend_Shortcode_Job extends Frontend_Shortcode {
    private $interfacer = null;
	/**
	 * @var array
	 */
	private $labels;

	public function __construct() {
        $this->interfacer = WP_Interfacer::getInstance();
    }

	public function register_shortcodes( $dataDictionary = array(), $categoryLanguage = false )
	{
		// new style (fx = attached function, section = defined in getSectionLabelKey() and have to match meta field key )
		$this->shortcodeDictionary["cx_intro_information"] = array( "fx" => "vacancy_information", "section" => "introInformation" );
		$this->shortcodeDictionary["cx_company_information"] = array( "fx" => "vacancy_information", "section" => "companyInformation" );
		$this->shortcodeDictionary["cx_vacancy_information"] = array( "fx" => "vacancy_information", "section" => "vacancyInformation" );
		$this->shortcodeDictionary["cx_requirements_information"] = array( "fx" => "vacancy_information", "section" => "requirementsInformation" );
		$this->shortcodeDictionary["cx_offer_information"] = array( "fx" => "vacancy_information", "section" => "offerInformation" );
		$this->shortcodeDictionary["cx_end_date"] = array( "fx" => "vacancy_information", "section" => "endDate" );
		$this->shortcodeDictionary["cx_function_contact_information"] = array( "fx" => "vacancy_information", "section" => "functionContactInformation" );
		$this->shortcodeDictionary["cx_application_contact_information"] = array( "fx" => "vacancy_information", "section" => "applicationContactInformation" );
		$this->shortcodeDictionary["cx_function_group"] = array( "fx" => "vacancy_information", "section" => "function0" );
		$this->shortcodeDictionary["cx_extra"] = array( "fx" => "vacancy_additional_field" );



		// old style
		$this->shortcodeDictionary["cx_apply_until"] = array( "fx" => "apply_until" );

		$this->shortcodeDictionary["cx_rss_for_category"] = array( "fx" => "rss_for_category" );
		$this->shortcodeDictionary["cx_rss_for_all"] = array( "fx" => "rss_for_all" );
		$this->shortcodeDictionary["cx_tell_a_friend_link"] = array( "fx" => "tell_a_friend_link" );
		$this->shortcodeDictionary["cx_print_link"] = array( "fx" => "print_link" );
		$this->shortcodeDictionary["cx_brief_section"] = array( "fx" => "brief_section" ); // unused
		// $this->shortcodeDictionary["cx_function_group"] = array( "fx" => "function_group_legacy" ); // Legacy: includes function0 + function1 and uses an API call
		$this->shortcodeDictionary["cx_vacancy_number"] = array( "fx" => "vacancy_number" );
		$this->shortcodeDictionary["cx_vacancy_code"] = array( "fx" => "vacancy_code" );
		$this->shortcodeDictionary["cx_social_icons"] = array( "fx" => "social_icons" );
		$this->shortcodeDictionary["cx_apply_button"] = array( "fx" => "apply_button" );
		$this->shortcodeDictionary["cx_job_alert_subscription_link"] = array( "fx" => "job_alert_subscription_link" );

		// Utils_Log::info("\n*register_shortcodes:*\n" . print_r($dataDictionary, true));

		parent::register_shortcodes( $dataDictionary );

		$this->labels = $this->getLabels( $categoryLanguage );
	}


	/**
	 * @param $atts
	 * @param $content
	 * @param $dataDictionary
	 *
	 * @return string
	 */
	public function vacancy_information( $atts, $content, $dataDictionary ) {
		$section_key = $atts['section_key']; // E.g. "introInformation";
		$value = $dataDictionary[$section_key] ?? get_post_meta(get_the_ID(), $section_key, true) ?? "";
		if ( strlen(trim($value)) == 0 ) return "";

//		Utils_Log::info("\n 1: " . print_r($atts, true) . "\n");

		$atts = shortcode_atts( array(
			'shortcode_name' => '',
			'section_key' => '',
			'label' => 'yes', // render the heading
			'heading' => 'h4',
			'content' => 'yes', // render the paragraph content
			'class' => '', // prepend custom css classes
			'quotes' => 'double', // legacy renderer uses single quotes for HTML attributes
			'raw' => 'no', // raw mode will leave out added html-tags
		), $atts );

//		Utils_Log::info("\n 2:" . print_r($atts, true) . "\n");


		$show_label = $this->yesNoToBool(trim($atts['label']));
		$show_content = $this->yesNoToBool(trim($atts['content']));
		$heading = (int) preg_replace('/[^0-9]+/', '', trim($atts['heading']));
		$attr_quote = ( trim(strtolower($atts['quotes'])) == "single" ) ? "'" : '"'; // support single quote for legacy purposes
		$show_raw = $this->yesNoToBool(trim($atts['raw']), false);

		// $contentSections = $this->getContentSections();
		$label_key = $this->getSectionLabelKey($section_key);
		$section_name = $this->labels[$label_key];
		$css_class = trim($atts['class'] . " cx2_" . @strtolower(@end(@explode(".", $label_key))));

		$content = "";
		/**
		 * Only show the <h1..x>-heading if label=yes and heading=h1|h2|h3|h4|h5|h6+
		 * Advantage: [cx_intro_information heading="no"] will drop the <h1..6>-tag!
		 */
		if ($show_label && $heading>0) {
			if ($show_raw) $content .= "{$section_name}\n";
			else $content .= "<h{$heading} class={$attr_quote}cx_h{$heading} {$css_class}{$attr_quote}>{$section_name}</h{$heading}>\n";
		}
		if ($show_content) {
			if ($show_raw) $content .= "{$value}\n";
			else $content .= "<div class={$attr_quote}{$css_class}{$attr_quote}>{$value}</div>\n";
		}

		// if ( $section_key == "introInformation" ) $content .= '<!--more-->';

		return $content;
	}


	public function vacancy_additional_field( $atts, $content, $dataDictionary ) : string {
		$additionalfieldssupport_status = Environment_Settings::getCxAdditionalFieldsSupportStatus();

		$fieldname = $atts['field'] ?? "";
		$fieldname = trim(strtolower($fieldname));
		$separator = $atts['separator'] ?? ", "; // checkboxes/multi-options are seperated

		if ($additionalfieldssupport_status == "off") {
			$content = "";
		} else if ( strlen($fieldname) == 0 ) {
			$extra_fields = $this->interfacer->db_get_additional_cx_fields();
			$content = "Shortcode error. ";
			if ( count($extra_fields) == 0) {
				$content .= "There are no correctly defined Additional Fields present in your Carerix System.";
			} else {
				$extra_fields = str_replace(WP_Constants::ADDITIONAL_FIELD_PREFIX, "", $extra_fields);
				$content .= "Empty or no fieldname supplied.<br>
							<br>
							Usage example: [{$atts['shortcode_name']} field=\"{$extra_fields[0]}\" separator=\", \"]<br>
							<br>
							The following additional fields are found: " . implode(", ", $extra_fields);
			}
			$content = "<div style='padding: 10px; border: red solid 2px;'>{$content}</div>";
		} else {
			$fieldname = preg_replace("/[^a-z0-9]+/", "_", $fieldname); // only allow a-z and digits. The rest is replaced by underscores.
			if ( strpos($fieldname, WP_Constants::ADDITIONAL_FIELD_PREFIX) !== 0 ) {
				$fieldname = WP_Constants::ADDITIONAL_FIELD_PREFIX . $fieldname;
			}

			/**
			 * First, try to fetch the value from the 'dataDictionary' . The dictionary is filled from the REST - API publication fetch.
			 * If that fails, it's a shortcode used on the front-end. For example in a theme template (do_shortcode()) or as user-input ([cx_extra...])
			 */
			$content = $dataDictionary[$fieldname] ?? get_post_meta(get_the_ID(), $fieldname) ?? "";

			if (is_array($content)) {
				$content = implode($separator, $content);
			}
		}
		return $content;
	}


	public function apply_until( $atts, $content, $dataDictionary ) {
		$date_publicationEnd = $dataDictionary["endDate"] ?? get_post_meta(get_the_ID(), "endDate", true) ?? "";
		if ( isset($date_publicationEnd) ) {
			$date_publicationEnd = mysql2date("Y-m-d H:m:s", $date_publicationEnd . " -1 day");
		}
		$date_deadlineMaterial = $dataDictionary["deadlineMaterial"] ?? get_post_meta(get_the_ID(), "deadlineMaterial", true) ?? "";
		$date_reactionEndDate = $dataDictionary["reactionEndDate"] ?? get_post_meta(get_the_ID(), "reactionEndDate", true) ?? "";

		$dates = array();
		if ( !empty($date_publicationEnd) ) $dates[] = $date_publicationEnd; // publication end date from cx publication
		if ( !empty($date_deadlineMaterial) ) $dates[] = $date_deadlineMaterial; // closing date from cx publication
		if ( !empty($date_reactionEndDate) ) $dates[] = $date_reactionEndDate; // closing date from cx job order
		sort($dates);

		if ( count($dates) ) {
			$the_date = mysql2date( get_option( 'date_format' ), $dates[0] );
		} else {
			$the_date = "";
		}

		return $the_date;
	}


    public function rss_for_category() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        
        $labels = Utils_Labels::get();
        $category = get_the_category();
        $category_name = '';
        $term_id = 0;

        if(isset($category[0])) {
            $term = $category[0];
            $term_id = $term->term_id;
            $category_name = $term->name ;
        }

        $feedgeneral = get_category_feed_link($term_id);

        $div = "<div class='rss'><a id='carerix_rss_link' target='_blank' rel='nofollow' href='{$feedgeneral}'>{$labels['CxPortal.message.field.rss']} $category_name</a></div>";
        echo $div;
        
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }

    public function rss_for_all() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        $labels = Utils_Labels::get();
        $categoryLanguage = $this->interfacer->get_default_site_language();
        $resource = 'resource_category_'. str_replace('.', '_', 'general.header.publications');
        $categoryKey = Environment_Settings::_setting_get($resource . '_' . $categoryLanguage);

        if(!$categoryKey) $categoryKey = $labels[$resource];
        $term = get_term_by('name', $categoryKey, 'category');

        if($term === false) {
            $terms = get_the_category();
            foreach($terms as $_term) {
                if($_term->category_parent === 0) {
                    $term = $_term;
                    break;
                }
            }
        }
        
        $url = get_category_feed_link(@$term->term_id);

        $div = "<div class='rss_all'><a id='carerix_rss_link_all' target='_blank' rel='nofollow' href='$url'>{$labels['CxPortal.message.field.rssAll']}</a></div>";
        echo $div;
        
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }
    
    public function tell_a_friend_link() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        $labels = Utils_Labels::get();
        $link = $this->interfacer->get_post_permalink(false, 'tellafriend');
        
        $div = "<div class='tell_a_friend'><a id='carerix_tellafriend_link' rel='nofollow' href='$link'>{$labels['CxPortal.header.tellAFriendTitle']}</a></div>";

		echo $div;
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }
    
    public function print_link() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        $labels = Utils_Labels::get();
        $div = "<div class='print'><a id='carerix_print_link' rel='nofollow' href='#' onclick='window.print();  '>{$labels['general.button.print']}</a></div>";	
        echo $div;
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }

	/**
	 * Vacancy Number
	 * Internal CX fieldname: vacancyID
	 *
	 * @return false|string|void
	 */
    public function vacancy_number() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        $labels = Utils_Labels::get();
        $post_id = $post->ID;
        $interfacer = WP_Interfacer::getInstance();
        $props = $interfacer->entity_get_properties($post_id);
        $vacancyID = (string)$props['vacancyID'][0];
        
        $div = "<div id='carerix_vacancy_id'><strong>{$labels['general.header.jobID']}:</strong> #{$vacancyID} </div>";
        echo $div;
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }

 	/**
	 * Vacancy reference code
	 * Internal CX fieldname: vacancyNo
	 *
	 * @return string
	 */
    public function vacancy_code() {
	    return get_post_meta(get_the_ID(), 'vacancyNo', true);
    }

    public function social_icons() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        $div = "<div class='social_icons'>
						<div id='cx_socials'>
							<span class='st_facebook' ></span><span class='st_twitter' ></span>					
                            <span class='st_linkedin' ></span><span class='st_email' ></span>
						</div>
                        <script type='text/javascript'>var switchTo5x=true;</script>
						<script type='text/javascript' src='https://ws.sharethis.com/button/buttons.js'></script>
                        <script type='text/javascript'>stLight.options({publisher: '9b61bbc7-5b60-40a2-9ea9-36c8b84e5406', doNotHash: false, doNotCopy: false, hashAddressBar: false});</script>
						
                    </div>";
        echo $div;
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }

	public function apply_button() {
		global $post;
		if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
			|| $post->post_type == 'page')
		{
			return;
		}
		$labels = Utils_Labels::get();
		$post_permalink = $this->interfacer->get_post_permalink(false, 'apply=true');
		$label = (isset($_GET['apply']) && $_GET['apply'] == 'true') ? $labels['general.button.applicationSubmit'] : $labels['general.button.apply'];
		return "<div><a id='apply_button' class='cx2_vacancydetails-apply' href='$post_permalink'>{$label}</a></div>";
	}
    
    public function job_alert_subscription_link() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        $link = $this->interfacer->get_post_permalink(false, 'subscribe');
        $labels = Utils_Labels::get();
        $div = "<div class='apply'><a id='carerix_jas_link' rel='nofollow' href='$link'>{$labels['general.button.subscribeJobAlert']}</a></div>";	
        echo $div;
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }
    
    public function function_group_legacy() {
        global $post;
        if(    $this->interfacer->frontend_displayed_single() === FALSE
			&& $this->interfacer->frontend_displayed_page() === FALSE
            || $post->post_type == 'page')
		{
			return;
		}
        ob_start();
        
        $post_id = $post->ID;

        $interfacer = WP_Interfacer::getInstance();
        $props = $interfacer->entity_get_properties($post_id);
        $pubKey = '_carerix_job';
        if($this->interfacer->wpml_is_active()) {
            $pubKey .= '_'.$this->interfacer->get_default_site_language();
        }
        
        $pubID = (string)$props[$pubKey][0];
        
        if($pubID) {
            $params = array('show' => array('toVacancy', 'toVacancy.toFunctionLevel1.value','toVacancy.toFunctionLevel2.value','toVacancy.toFunctionLevel3.value', 'publicationID') );

            $publication = CRPublication::find($pubID, $params);
            $labels = Utils_Labels::get();

            $functionGroup = implode(' / ', array_filter(array(
                            $publication->toVacancy->toFunctionLevel1->value,
                            $publication->toVacancy->toFunctionLevel2->value,
                            $publication->toVacancy->toFunctionLevel3->value
                        )));
            $div = '';
            if($functionGroup) $div = "<div id='carerix_function_group'><strong>{$labels['CxPortal.message.field.functionGroup']}:</strong> $functionGroup</div>";
            echo $div;
        }
        
        // $out = ob_get_contents();
        $out = ob_get_clean();
        return $out;
    }




	public function brief_section() { }
    
//    static function get_source_data() {
//        global $post;
//        if(    $this->interfacer->frontend_displayed_single() === FALSE
//			&& $this->interfacer->frontend_displayed_page() === FALSE
//            || $post->post_type == 'page')
//		{
//			return;
//		}
//
//        $post_id = $post->ID;
//        $interfacer = WP_Interfacer::getInstance();
//        $props = $interfacer->entity_get_properties($post_id);
//        $source_id = (string)$props['source'][0];
//        $source = Table_Source::get();
//        $source_data = $source->getByID($source_id);
//        $source_data['settings'] = unserialize($source_data['settings']);
//        return $source_data;
//    }

	/**
	 * @return array
	 */
	public function getLabels( $categoryLanguage = false )
	{
		if(isset($this->dataDictionary['medium'])) {
			$_l = explode('-', $this->dataDictionary['medium']);
			if(isset($_l[1])) {
				$language = trim($_l[1]);

				$language .= '-' . strtoupper($language);
				$labels = Utils_Labels::get($language);

			} else {
				$labels = Utils_Labels::get();
			}
		} else {
			$labels = Utils_Labels::get($categoryLanguage);
		}

		return $labels;
	}

	/**
	 * @return string[]
	 */
	public function getContentSections() {
		$contentSections = array(
			'introInformation',
			'companyInformation',
			'vacancyInformation',
			'requirementsInformation',
			'offerInformation',
			'endDate',
			'functionContactInformation',
			'applicationContactInformation',
		);

		return $contentSections;
	}


	/**
	 * @param $section
	 *
	 * @return string
	 */
	public function getSectionLabelKey( $section ): string {
		$labelKeysForSection = array(
			'introInformation' => 'CxPortal.message.field.introduction',
			'companyInformation' => 'CxPortal.message.field.organization',
			'vacancyInformation' => 'CxPortal.message.field.function',
			'requirementsInformation' => 'CxPortal.message.field.requirements',
			'offerInformation' => 'CxPortal.message.field.offer',
			'endDate' => 'CxPortal.message.field.closingDate',
			'function0' => 'CxPortal.message.field.singleFunctionGroup',
			'functionContactInformation' => 'CxPortal.message.field.information',
			'applicationContactInformation' => 'CxPortal.message.field.application',
		);

		return $labelKeysForSection[$section];
	}

	private function yesNoToBool(string $value, bool $default = true)
	{
		$value = trim(strtolower($value));
		switch ($value) {
			case "yes":
			case "1":
			case "true":
			case "on":
			case "show":
				$return_val = true;
				break;
			case "no":
			case "0":
			case "false":
			case "off":
			case "hide":
				$return_val = false;
				break;
			default:
				$return_val = $default;
		}

		return $return_val;
	}

}

