<?php /** @noinspection PhpUndefinedFunctionInspection */


namespace Carerix\Captcha;

use Gregwar\Captcha\CaptchaBuilder;

/**
 * Creates, displays and matches a captcha image
 *
 * @author Ruben <ruben@carerix.com>
 *
 * @uses CaptchaBuilder
 */
class Captcha
{
	public static function testCaptchaMatch()
	{
		$captcha_user_input = isset($_POST['captcha_code']) ? trim(strtolower($_POST['captcha_code'])) : "";
		$captcha_user_input_nonce = wp_create_nonce( 'captcha_' . $captcha_user_input );

		$captcha_expected_nonce = isset($_POST['captcha_expected_nonce']) ? trim(strtolower($_POST['captcha_expected_nonce'])) : "";

		if ( strlen($captcha_user_input) && $captcha_user_input_nonce == $captcha_expected_nonce ) {
			wp_cache_set('captcha_match', 1, CARERIX_CACHE_GROUP);
		} else {
			wp_cache_set('captcha_match', 0, CARERIX_CACHE_GROUP);
		}
	}

	public static function buildNewCaptcha()
	{
		$captcha_builder = new CaptchaBuilder;
//		$captcha_builder->setMaxBehindLines(0);
//		$captcha_builder->setMaxFrontLines(0);
		$captcha_builder->setIgnoreAllEffects(1);
		$captcha_builder->build();

		$captcha_code = strtolower($captcha_builder->getPhrase());
		wp_cache_set('captcha_code', $captcha_code, CARERIX_CACHE_GROUP);
		$captcha_inlineimage = $captcha_builder->inline();
		wp_cache_set('captcha_inlineimage', $captcha_inlineimage, CARERIX_CACHE_GROUP);
		$captcha_expected_nonce = wp_create_nonce( 'captcha_' . $captcha_code );
		wp_cache_set('captcha_expected_nonce', $captcha_expected_nonce, CARERIX_CACHE_GROUP);
	}

	public static function displayCaptcha()
	{
		$nonce = self::getExpectedNonce();
		$inline_img = self::getInlineImage();

		return <<<EOT
<input type="text" name="captcha_code" value="" autocomplete="off" class="carerix_required" /> <br />
<input type="hidden" name="captcha_expected_nonce" value="{$nonce}" />
<img src="{$inline_img}" alt="CAPTCHA code">
EOT;
	}

	/**
	 * Get the results of the captcha test
	 *
	 * @return bool
	 */
	public static function hasUserPassedTheTest()
	{
		return wp_cache_get('captcha_match', CARERIX_CACHE_GROUP) === 0 ? false : true;
	}


	private static function getInlineImage()
	{
		return wp_cache_get('captcha_inlineimage', CARERIX_CACHE_GROUP);
	}

	private static function getExpectedNonce()
	{
		return wp_cache_get('captcha_expected_nonce', CARERIX_CACHE_GROUP);
	}


}